@@ -1,7 +1,7 @@
 // This module defines the "official" high-level API of the Qt port of
 // Scintilla.
 //
-// Copyright (c) 2017 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2019 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
 // This file is part of QScintilla.
 // 
@@ -302,6 +302,14 @@ public:
         //! A triangle below the centre of the first character in the indicator
         //! range.
         TriangleCharacterIndicator = INDIC_POINTCHARACTER,
+
+        //! A vertical gradient between the indicator's foreground colour at
+        //! top to fully transparent at the bottom.
+        GradientIndicator = INDIC_GRADIENT,
+
+        //! A vertical gradient with the indicator's foreground colour in the
+        //! middle and fading to fully transparent at the top and bottom.
+        CentreGradientIndicator = INDIC_GRADIENTCENTRE,
     };
 
     //! This enum defines the different margin options.
@@ -508,7 +516,11 @@ public:
 
         //! Wrapped sub-lines are indented by the same amount as the first
         //! sub-line plus one more level of indentation.
-        WrapIndentIndented = SC_WRAPINDENT_INDENT
+        WrapIndentIndented = SC_WRAPINDENT_INDENT,
+
+        //! Wrapped sub-lines are indented by the same amount as the first
+        //! sub-line plus two more level of indentation.
+        WrapIndentDeeplyIndented = SC_WRAPINDENT_DEEPINDENT
     };
 
     //! Construct an empty QsciScintilla with parent \a parent.
@@ -640,6 +652,10 @@ public:
     //! \sa setCallTipsVisible()
     int callTipsVisible() const {return maxCallTips;}
 
+    //! Cancel any previous call to findFirst(), findFirstInSelection() or
+    //! findNext() so that replace() does nothing.
+    void cancelFind();
+
     //! Cancel any current auto-completion or user defined list.
     void cancelList();
 
@@ -779,10 +795,13 @@ public:
     //! POSIX compatible manner by interpreting bare ( and ) as tagged sections
     //! rather than \( and \).
     //!
-    //! \sa findFirstInSelection(), findNext(), replace()
+    //! If \a cxx11 is true then a regular expression is treated as a Cxx11
+    //! regular expression.
+    //!
+    //! \sa cancelFind(), findFirstInSelection(), findNext(), replace()
     virtual bool findFirst(const QString &expr, bool re, bool cs, bool wo,
             bool wrap, bool forward = true, int line = -1, int index = -1,
-            bool show = true, bool posix = false);
+            bool show = true, bool posix = false, bool cxx11 = false);
 
     //! Find the first occurrence of the string \a expr in the current
     //! selection and return true if \a expr was found, otherwise returns
@@ -809,15 +828,18 @@ public:
     //! POSIX compatible manner by interpreting bare ( and ) as tagged sections
     //! rather than \( and \).
     //!
-    //! \sa findFirstInSelection(), findNext(), replace()
+    //! If \a cxx11 is true then a regular expression is treated as a Cxx11
+    //! regular expression.
+    //!
+    //! \sa cancelFind(), findFirst(), findNext(), replace()
     virtual bool findFirstInSelection(const QString &expr, bool re, bool cs,
             bool wo, bool forward = true, bool show = true,
-            bool posix = false);
+            bool posix = false, bool cxx11 = false);
 
     //! Find the next occurence of the string found using findFirst() or
     //! findFirstInSelection().
     //!
-    //! \sa findFirst(), findFirstInSelection(), replace()
+    //! \sa cancelFind(), findFirst(), findFirstInSelection(), replace()
     virtual bool findNext();
 
     //! Returns the number of the first visible line.
@@ -1137,7 +1159,7 @@ public:
     //! Replace the current selection, set by a previous call to findFirst(),
     //! findFirstInSelection() or findNext(), with \a replaceStr.
     //!
-    //! \sa findFirst(), findFirstInSelection(), findNext()
+    //! \sa cancelFind(), findFirst(), findFirstInSelection(), findNext()
     virtual void replace(const QString &replaceStr);
 
     //! Reset the fold margin colours to their defaults.
@@ -1155,6 +1177,16 @@ public:
     //! \sa setHotspotForegroundColor(), resetHotspotBackgroundColor()
     void resetHotspotForegroundColor();
 
+    //! Gets the assumed document width in pixels.
+    //!
+    //! \sa setScrollWidth(), setScrollWidthTracking()
+    int scrollWidth() const;
+
+    //! Returns true if scroll width tracking is enabled.
+    //!
+    //! \sa scrollWidth(), setScrollWidthTracking()
+    bool scrollWidthTracking() const;
+
     //! The fold margin may be drawn as a one pixel sized checkerboard pattern
     //! of two colours, \a fore and \a back.
     //!
@@ -1370,6 +1402,23 @@ public:
     //! \sa setMatchedBraceIndicator()
     void resetMatchedBraceIndicator();
 
+    //! For performance, QScintilla does not measure the display width of the
+    //! document to determine the properties of the horizontal scroll bar.
+    //! Instead, an assumed width is used.  This sets the document width in
+    //! pixels assumed by QScintilla to \a pixelWidth.  The default value is
+    //! 2000.
+    //!
+    //! \sa scrollWidth(), setScrollWidthTracking()
+    void setScrollWidth(int pixelWidth);
+
+    //! If scroll width tracking is enabled then the scroll width is adjusted
+    //! to ensure that all of the lines currently displayed can be completely
+    //! scrolled.  This mode never adjusts the scroll width to be narrower.
+    //! This sets the scroll width tracking to \a enabled.
+    //!
+    //! \sa setScrollWidth(), scrollWidthTracking()
+    void setScrollWidthTracking(bool enabled);
+
     //! Sets the mode used to draw tab characters when whitespace is visible to
     //! \a mode.  The default is to use an arrow.
     //!
@@ -1763,6 +1812,10 @@ public slots:
     //! \sa setCaretLineVisible()
     virtual void setCaretLineBackgroundColor(const QColor &col);
 
+    //! Sets the width of the frame of the line containing the caret to \a
+    //! width.
+    virtual void setCaretLineFrameWidth(int width);
+
     //! Enables or disables, according to \a enable, the background color of
     //! the line containing the caret.
     //!
@@ -2009,34 +2062,6 @@ public slots:
     //! \sa zoomIn(), zoomOut()
     virtual void zoomTo(int size);
 
-    //! For performance, Scintilla does not measure the display width
-    //! of the document to determine the properties of the horizontal
-    //! scroll bar. Instead, an assumed width is used.  This sets the
-    //! document width in pixels assumed by Scintilla to \a
-    //! pixelWidth. The default value is 2000.
-    //!
-    //! \sa getScrollWidth(), setScrollWidthTracking()
-    virtual void setScrollWidth(int pixelWidth);
-
-    //! Gets the document width in pixels assumed by Scintilla.
-    //!
-    //! \sa setScrollWidth(), setScrollWidthTracking()
-    virtual int getScrollWidth() const;
-
-    //! If scroll width tracking is enabled then the scroll width is
-    //! adjusted to ensure that all of the lines currently displayed
-    //! can be completely scrolled. This mode never adjusts the scroll
-    //! width to be narrower.
-    //! Sets the scroll width tracking to \a enabled.
-    //!
-    //! \sa setScrollWidth(), getScrollWidthTracking()
-    virtual void setScrollWidthTracking(bool enabled);
-
-    //! Gets the current scroll width tracking mode.
-    //!
-    //! \sa getScrollWidth(), setScrollWidthTracking()
-    virtual bool getScrollWidthTracking() const;
-
 signals:
     //! This signal is emitted whenever the cursor position changes.  \a line
     //! contains the line number and \a index contains the character index
@@ -2128,6 +2153,9 @@ protected:
     //! \reimp
     virtual void contextMenuEvent(QContextMenuEvent *e);
 
+    //! \reimp
+    virtual void wheelEvent(QWheelEvent *e);
+
 private slots:
     void handleCallTipClick(int dir);
     void handleCharAdded(int charadded);
