@@ -5,7 +5,7 @@
 
 #include "CipherSettings.h"
 
-class QRegExpValidator;
+class QRegularExpressionValidator;
 
 namespace Ui {
 class CipherDialog;
@@ -19,14 +19,14 @@ public:
     // Set the encrypt parameter to true when the dialog is used to encrypt a database;
     // set it to false if the dialog is used to ask the user for the key to decrypt a file.
     explicit CipherDialog(QWidget* parent, bool encrypt);
-    ~CipherDialog();
+    ~CipherDialog() override;
 
     CipherSettings getCipherSettings() const;
 
 private:
     Ui::CipherDialog* ui;
     bool encryptMode;
-    QRegExpValidator* rawKeyValidator;
+    QRegularExpressionValidator* rawKeyValidator;
 
 private slots:
     void checkInputFields();
