@@ -1,11 +1,16 @@
+#include <QMessageBox>
+
 #include "ColumnDisplayFormatDialog.h"
 #include "ui_ColumnDisplayFormatDialog.h"
-#include "sqlitetypes.h"
+#include "sql/sqlitetypes.h"
+#include "sqlitedb.h"
 
-ColumnDisplayFormatDialog::ColumnDisplayFormatDialog(const QString& colname, QString current_format, QWidget* parent)
+ColumnDisplayFormatDialog::ColumnDisplayFormatDialog(DBBrowserDB& db, const sqlb::ObjectIdentifier& tableName, const QString& colname, const QString& current_format, QWidget* parent)
     : QDialog(parent),
       ui(new Ui::ColumnDisplayFormatDialog),
-      column_name(colname)
+      column_name(colname),
+      pdb(db),
+      curTable(tableName)
 {
     // Create UI
     ui->setupUi(this);
@@ -20,6 +25,7 @@ ColumnDisplayFormatDialog::ColumnDisplayFormatDialog(const QString& colname, QSt
     ui->comboDisplayFormat->insertSeparator(ui->comboDisplayFormat->count());
     ui->comboDisplayFormat->addItem(tr("Apple NSDate to date"), "appleDate");
     ui->comboDisplayFormat->addItem(tr("Java epoch (milliseconds) to date"), "javaEpoch");
+    ui->comboDisplayFormat->addItem(tr(".NET DateTime.Ticks to date"), "dotNetTicks");
     ui->comboDisplayFormat->addItem(tr("Julian day to date"), "julian");
     ui->comboDisplayFormat->addItem(tr("Unix epoch to date"), "epoch");
     ui->comboDisplayFormat->addItem(tr("Unix epoch to local time"), "epochLocalTime");
@@ -28,6 +34,9 @@ ColumnDisplayFormatDialog::ColumnDisplayFormatDialog(const QString& colname, QSt
     ui->comboDisplayFormat->insertSeparator(ui->comboDisplayFormat->count());
     ui->comboDisplayFormat->addItem(tr("Lower case"), "lower");
     ui->comboDisplayFormat->addItem(tr("Upper case"), "upper");
+    ui->comboDisplayFormat->insertSeparator(ui->comboDisplayFormat->count());
+    ui->comboDisplayFormat->addItem(tr("Custom"), "custom");
+
     ui->labelDisplayFormat->setText(ui->labelDisplayFormat->text().arg(column_name));
 
     formatFunctions["decimal"] = "printf('%d', " + sqlb::escapeIdentifier(column_name) + ")";
@@ -39,6 +48,7 @@ ColumnDisplayFormatDialog::ColumnDisplayFormatDialog(const QString& colname, QSt
     formatFunctions["appleDate"] = "datetime('2001-01-01', " + sqlb::escapeIdentifier(column_name) + " || ' seconds')";
     formatFunctions["javaEpoch"] = "strftime('%Y-%m-%d %H:%M:%S.', " + sqlb::escapeIdentifier(column_name) +
             "/1000, 'unixepoch') || (" + sqlb::escapeIdentifier(column_name) + "%1000)";
+    formatFunctions["dotNetTicks"] = "datetime(" + sqlb::escapeIdentifier(column_name) + " / 10000000 - 62135596800, 'unixepoch')";
     formatFunctions["julian"] = "datetime(" + sqlb::escapeIdentifier(column_name) + ")";
     formatFunctions["epoch"] = "datetime(" + sqlb::escapeIdentifier(column_name) + ", 'unixepoch')";
     formatFunctions["epochLocalTime"] = "datetime(" + sqlb::escapeIdentifier(column_name) + ", 'unixepoch', 'localtime')";
@@ -53,19 +63,13 @@ ColumnDisplayFormatDialog::ColumnDisplayFormatDialog(const QString& colname, QSt
         ui->comboDisplayFormat->setCurrentIndex(0);
         updateSqlCode();
     } else {
-        QString formatName;
-        for(auto& formatKey : formatFunctions.keys()) {
-            if(current_format == formatFunctions.value(formatKey)) {
-                formatName = formatKey;
-                break;
-            }
-        }
-
-        if(formatName.isEmpty()) {
-            ui->comboDisplayFormat->insertSeparator(ui->comboDisplayFormat->count());
-            ui->comboDisplayFormat->addItem(tr("Custom"), "custom");
-            formatName = "custom";
-        }
+        // When it doesn't match any predefined format, it is considered custom
+        QString formatName = "custom";
+        auto it = std::find_if(formatFunctions.begin(), formatFunctions.end(), [current_format](const std::pair<std::string, QString>& s) {
+            return s.second == current_format;
+        });
+        if(it != formatFunctions.end())
+            formatName = QString::fromStdString(it->first);
         ui->comboDisplayFormat->setCurrentIndex(ui->comboDisplayFormat->findData(formatName));
         ui->editDisplayFormat->setText(current_format);
     }
@@ -86,11 +90,50 @@ QString ColumnDisplayFormatDialog::selectedDisplayFormat() const
 
 void ColumnDisplayFormatDialog::updateSqlCode()
 {
-    QString format = ui->comboDisplayFormat->currentData().toString();
+    std::string format = ui->comboDisplayFormat->currentData().toString().toStdString();
 
     if(format == "default")
         ui->editDisplayFormat->setText(sqlb::escapeIdentifier(column_name));
+    else if(format != "custom")
+        ui->editDisplayFormat->setText(formatFunctions.at(format));
+}
+
+void ColumnDisplayFormatDialog::accept()
+{
+    QString errorMessage;
+
+    // Accept the SQL code if it's the column name (default), it contains a function invocation applied to the column name and it can be
+    // executed without errors returning only one column.
+    // Users could still devise a way to break this, but this is considered good enough for letting them know about simple incorrect
+    // cases.
+    if(!(ui->editDisplayFormat->text() == sqlb::escapeIdentifier(column_name) ||
+         ui->editDisplayFormat->text().contains(QRegularExpression("[a-z]+[a-z_0-9]* *\\(.*" + QRegularExpression::escape(sqlb::escapeIdentifier(column_name)) + ".*\\)", QRegularExpression::CaseInsensitiveOption))))
+        errorMessage = tr("Custom display format must contain a function call applied to %1").arg(sqlb::escapeIdentifier(column_name));
+    else {
+        // Execute a query using the display format and check that it only returns one column.
+        int customNumberColumns = 0;
+
+        DBBrowserDB::execCallback callback = [&customNumberColumns](int numberColumns, std::vector<QByteArray>, std::vector<QByteArray>) -> bool {
+            customNumberColumns = numberColumns;
+            // Return false so the query is not aborted and no error is reported.
+            return false;
+        };
+        if(!pdb.executeSQL("SELECT " + ui->editDisplayFormat->text().toStdString() + " FROM " + curTable.toString() + " LIMIT 1",
+                           false, true, callback))
+            errorMessage = tr("Error in custom display format. Message from database engine:\n\n%1").arg(pdb.lastError());
+        else if(customNumberColumns != 1)
+            errorMessage = tr("Custom display format must return only one column but it returned %1.").arg(customNumberColumns);
+
+    }
+    if(!errorMessage.isEmpty())
+        QMessageBox::warning(this, QApplication::applicationName(), errorMessage);
     else
-        ui->editDisplayFormat->setText(formatFunctions.value(format));
+        QDialog::accept();
+}
 
+void ColumnDisplayFormatDialog::setCustom(bool modified)
+{
+    // If the SQL code is modified by user, select the custom value in the combo-box
+    if(modified && ui->editDisplayFormat->hasFocus())
+        ui->comboDisplayFormat->setCurrentIndex(ui->comboDisplayFormat->findData("custom"));
 }
