@@ -6,11 +6,15 @@
 #include <QMenu>
 #include <QWhatsThis>
 
-FilterLineEdit::FilterLineEdit(QWidget* parent, QList<FilterLineEdit*>* filters, int columnnum) : QLineEdit(parent), filterList(filters), columnNumber(columnnum)
+FilterLineEdit::FilterLineEdit(QWidget* parent, std::vector<FilterLineEdit*>* filters, size_t columnnum) :
+    QLineEdit(parent),
+    filterList(filters),
+    columnNumber(columnnum),
+    conditional_format(true)
 {
     setPlaceholderText(tr("Filter"));
     setClearButtonEnabled(true);
-    setProperty("column", columnnum);            // Store the column number for later use
+    setProperty("column", static_cast<int>(columnnum));            // Store the column number for later use
 
     // Introduce a timer for delaying the signal triggered whenever the user changes the filter value.
     // The idea here is that the textChanged() event isn't connected to the update filter slot directly anymore
@@ -19,8 +23,8 @@ FilterLineEdit::FilterLineEdit(QWidget* parent, QList<FilterLineEdit*>* filters,
     // delayedSignalTimerTriggered() method which then stops the timer and emits the delayed signal.
     delaySignalTimer = new QTimer(this);
     delaySignalTimer->setInterval(Settings::getValue("databrowser", "filter_delay").toInt());  // This is the milliseconds of not-typing we want to wait before triggering
-    connect(this, SIGNAL(textChanged(QString)), delaySignalTimer, SLOT(start()));
-    connect(delaySignalTimer, SIGNAL(timeout()), this, SLOT(delayedSignalTimerTriggered()));
+    connect(this, &FilterLineEdit::textChanged, delaySignalTimer, static_cast<void (QTimer::*)()>(&QTimer::start));
+    connect(delaySignalTimer, &QTimer::timeout, this, &FilterLineEdit::delayedSignalTimerTriggered);
 
     setWhatsThis(tr("These input fields allow you to perform quick filters in the currently selected table.\n"
                     "By default, the rows containing the input text are filtered out.\n"
@@ -32,15 +36,16 @@ FilterLineEdit::FilterLineEdit(QWidget* parent, QList<FilterLineEdit*>* filters,
                     "<=\tEqual to or less\n"
                     "=\tEqual to: exact match\n"
                     "<>\tUnequal: exact inverse match\n"
-                    "x~y\tRange: values between x and y"));
+                    "x~y\tRange: values between x and y\n"
+                    "/regexp/\tValues matching the regular expression"));
 
     // Immediately emit the delayed filter value changed signal if the user presses the enter or the return key or
     // the line edit widget loses focus
-    connect(this, SIGNAL(editingFinished()), this, SLOT(delayedSignalTimerTriggered()));
+    connect(this, &FilterLineEdit::editingFinished, this, &FilterLineEdit::delayedSignalTimerTriggered);
 
     // Prepare for adding the What's This information and filter helper actions to the context menu
     setContextMenuPolicy(Qt::CustomContextMenu);
-    connect(this, SIGNAL(customContextMenuRequested(const QPoint&)), this, SLOT(showContextMenu(const QPoint &)));
+    connect(this, &FilterLineEdit::customContextMenuRequested, this, &FilterLineEdit::showContextMenu);
 }
 
 void FilterLineEdit::delayedSignalTimerTriggered()
@@ -64,13 +69,13 @@ void FilterLineEdit::keyReleaseEvent(QKeyEvent* event)
 {
     if(event->key() == Qt::Key_Tab)
     {
-        if(columnNumber < filterList->size() - 1)
+        if(filterList && columnNumber < filterList->size() - 1)
         {
             filterList->at(columnNumber + 1)->setFocus();
             event->accept();
         }
     } else if(event->key() == Qt::Key_Backtab) {
-        if(columnNumber > 0)
+        if(filterList && columnNumber > 0)
         {
             filterList->at(columnNumber - 1)->setFocus();
             event->accept();
@@ -94,11 +99,11 @@ void FilterLineEdit::setText(const QString& text)
     delayedSignalTimerTriggered();
 }
 
-void FilterLineEdit::setFilterHelper(const QString& filterOperator)
+void FilterLineEdit::setFilterHelper(const QString& filterOperator, const QString& operatorSuffix)
 {
-    setText(filterOperator + "?");
+    setText(filterOperator + "?" + operatorSuffix);
     // Select the value for easy editing of the expression
-    setSelection(filterOperator.length(), filterOperator.length());
+    setSelection(filterOperator.length(), 1);
 }
 
 void FilterLineEdit::showContextMenu(const QPoint &pos)
@@ -134,7 +139,11 @@ void FilterLineEdit::showContextMenu(const QPoint &pos)
     connect(isNotEmptyAction, &QAction::triggered, [&]() {
             setText("<>''");
         });
-
+    // Simplify this if we ever support a NOT LIKE filter
+    QAction* notContainingAction = new QAction(tr("Not containing..."), editContextMenu);
+    connect(notContainingAction, &QAction::triggered, [&]() {
+            setFilterHelper(QString ("/^((?!"), QString(").)*$/"));
+        });
     QAction* equalToAction = new QAction(tr("Equal to..."), editContextMenu);
     connect(equalToAction, &QAction::triggered, [&]() {
             setFilterHelper(QString ("="));
@@ -164,6 +173,37 @@ void FilterLineEdit::showContextMenu(const QPoint &pos)
             setFilterHelper(QString ("?~"));
         });
 
+    QAction* regexpAction = new QAction(tr("Regular expression..."), editContextMenu);
+    connect(regexpAction, &QAction::triggered, [&]() {
+            setFilterHelper(QString ("/"), QString ("/"));
+        });
+
+
+    if(conditional_format)
+    {
+        QAction* conditionalFormatAction;
+        if (text().isEmpty()) {
+            conditionalFormatAction = new QAction(QIcon(":/icons/clear_cond_formats"), tr("Clear All Conditional Formats"), editContextMenu);
+            connect(conditionalFormatAction, &QAction::triggered, [&]() {
+                    emit clearAllCondFormats();
+            });
+        } else {
+            conditionalFormatAction = new QAction(QIcon(":/icons/add_cond_format"), tr("Use for Conditional Format"), editContextMenu);
+            connect(conditionalFormatAction, &QAction::triggered, [&]() {
+                    emit addFilterAsCondFormat(text());
+            });
+        }
+        QAction* editCondFormatsAction = new QAction(QIcon(":/icons/edit_cond_formats"), tr("Edit Conditional Formats..."), editContextMenu);
+        connect(editCondFormatsAction, &QAction::triggered, [&]() {
+            emit editCondFormats();
+        });
+        editContextMenu->addSeparator();
+
+        editContextMenu->addAction(conditionalFormatAction);
+        editContextMenu->addAction(editCondFormatsAction);
+    }
+
+
     filterMenu->addAction(whatsThisAction);
     filterMenu->addSeparator();
     filterMenu->addAction(isNullAction);
@@ -171,6 +211,7 @@ void FilterLineEdit::showContextMenu(const QPoint &pos)
     filterMenu->addAction(isEmptyAction);
     filterMenu->addAction(isNotEmptyAction);
     filterMenu->addSeparator();
+    filterMenu->addAction(notContainingAction);
     filterMenu->addAction(equalToAction);
     filterMenu->addAction(notEqualToAction);
     filterMenu->addAction(greaterThanAction);
@@ -178,6 +219,6 @@ void FilterLineEdit::showContextMenu(const QPoint &pos)
     filterMenu->addAction(greaterEqualAction);
     filterMenu->addAction(lessEqualAction);
     filterMenu->addAction(inRangeAction);
-
+    filterMenu->addAction(regexpAction);
     editContextMenu->exec(mapToGlobal(pos));
 }
