@@ -3,7 +3,6 @@
 
 #include <QDialog>
 
-class RemoteDatabase;
 class QRegExpValidator;
 
 namespace Ui {
@@ -15,14 +14,16 @@ class RemotePushDialog : public QDialog
     Q_OBJECT
 
 public:
-    RemotePushDialog(QWidget* parent, RemoteDatabase& remote, const QString& host, const QString& clientCert, const QString& name = QString());
-    ~RemotePushDialog();
+    explicit RemotePushDialog(QWidget* parent, const QString& host, const QString& clientCert,
+                              const QString& name = QString(), const QString& branch = QString(), const QString& user = QString());
+    ~RemotePushDialog() override;
 
     QString name() const;
     QString commitMessage() const;
     QString licence() const;
     bool isPublic() const;
     QString branch() const;
+    QString user() const;
     bool forcePush() const;
 
 private:
@@ -32,21 +33,14 @@ private:
     QString m_host;
     QString m_clientCert;
 
-    // Reference to the remote database object which is stored somewhere in the main window
-    RemoteDatabase& remoteDatabase;
-
     // Validators
     QRegExpValidator* m_nameValidator;
     QRegExpValidator* m_branchValidator;
 
 protected slots:
     void checkInput();
-    virtual void accept();
-
-    void reloadBranchList();
-
-    void fillInLicences(const QMap<QString, QString>& licences);
-    void fillInBranches(const QStringList& branches, const QString& default_branch);
+    void reloadBranchList(const QString& select_branch = QString());
+    void accept() override;
 };
 
 #endif
