@@ -3,7 +3,7 @@
 
 #include "Qsci/qscilexersql.h"
 
-#include <QMap>
+#include <map>
 
 class QsciAPIs;
 
@@ -23,16 +23,16 @@ public:
         ApiCompleterIconIdSchema,
     };
 
-    typedef QMap<QString, QList<QString> > TablesAndColumnsMap;
-    typedef QMap<QString, TablesAndColumnsMap > QualifiedTablesMap;
+    using TablesAndColumnsMap = std::map<QString, std::vector<QString>>;
+    using QualifiedTablesMap = std::map<QString, TablesAndColumnsMap>;
 
     void setTableNames(const QualifiedTablesMap& tables);
 
-    virtual const char* keywords(int set) const;
+    const char* keywords(int set) const override;
 
-    virtual QStringList autoCompletionWordSeparators() const;
+    QStringList autoCompletionWordSeparators() const override;
 
-    bool caseSensitive() const;
+    bool caseSensitive() const override;
 
 private:
     QsciAPIs* autocompleteApi;
@@ -41,6 +41,7 @@ private:
 
     QStringList listTables;
     QStringList listFunctions;
+    QStringList keywordPatterns;
 };
 
 #endif
