@@ -2,8 +2,9 @@
 #define DOCKTEXTEDIT_H
 
 #include "ExtendedScintilla.h"
-#include <Qsci/qscilexerjson.h>
-#include <Qsci/qscilexerxml.h>
+
+class QsciLexerJSON;
+class QsciLexerXML;
 
 /**
  * @brief The DockTextEdit class
@@ -15,18 +16,24 @@ class DockTextEdit : public ExtendedScintilla
 
 public:
     explicit DockTextEdit(QWidget *parent = nullptr);
-    virtual ~DockTextEdit();
 
     // Enumeration of supported languages
     enum Language
     {
+        PlainText,
         JSON,
         XML
     };
 
     void setLanguage(Language lang);
-    Language language() { return m_language; };
-    
+    Language language() const { return m_language; }
+
+    // Disables the line-number margin and sets this text in the first line.
+    void setTextInMargin(const QString& text);
+
+    // Resets margin to their original line-number mode
+    void clearTextInMargin();
+
 public slots:
     void reloadSettings();
 
@@ -35,6 +42,7 @@ protected:
     static QsciLexerXML* xmlLexer;
 private:
     Language m_language;
+    QFont plainTextFont;
 };
 
 #endif
