@@ -35,6 +35,13 @@ private slots:
     void createTableWithNotLikeConstraint();
     void rowValues();
     void complexExpressions();
+    void datetimeExpression();
+    void extraParentheses();
+    void moduloOperator();
+    void complexExpression();
+
+    void parseTest();
+    void parseTest_data();
 };
 
 #endif
